﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel.Design;
using System.Workflow.Activities;
using System.Workflow.ComponentModel.Design;
using System.Workflow.ComponentModel;
using System.ComponentModel;

namespace WorkflowDesigner.Workflow
{
    /// <summary>
    /// Logique d'interaction pour WorkflowDesigner.xaml
    /// </summary>
    public partial class WorkflowDesigner : UserControl, IServiceProvider
    {

        private DesignSurface designSurface;
        private WorkflowView workflowView;
        private WorkflowLoader loader;

        public WorkflowDesigner()
        {
            InitializeComponent();
        }

        public void AddActivity(Activity activity)
        {
            if (activity == null)
                throw new ArgumentNullException("activity");
            
            IDesignerHost designerHost = designSurface.GetService(typeof(IDesignerHost)) as IDesignerHost;
            if (designerHost == null)
                return;

            SequentialWorkflowActivity rootActivity = this.workflowView.RootDesigner.Activity as SequentialWorkflowActivity;
            if (rootActivity == null)
                return;

            //Si l'activité que l'on ajoute n'a pas de parent on la lie à l'activité root du designer
            if (activity.Parent == null)
                rootActivity.Activities.Add(activity);
            //On ajoute la nouvelle activité dans le designer
            AddActivityToDesigner(designerHost, activity);
        }

        private void AddActivityToDesigner(IDesignerHost designerHost, Activity activity)
        {
            if (activity == null)
                return;

            designerHost.Container.Add(activity);
            if (activity is CompositeActivity)
            {
                foreach (Activity child in ((CompositeActivity)activity).Activities)
                    AddActivityToDesigner(designerHost, child);
            }
        }

        public void LoadWorflowDesigner()
        {
            loader = new WorkflowLoader();

            this.designSurface = new DesignSurface();
            this.designSurface.BeginLoad(loader);

            //Récupération du designer
            IDesignerHost designerHost = designSurface.GetService(typeof(IDesignerHost)) as IDesignerHost;
            if (designerHost == null)
                return;
            IRootDesigner rootDesigner = designerHost.GetDesigner(designerHost.RootComponent) as IRootDesigner;
            if (rootDesigner == null)
                return;

            this.workflowView = rootDesigner.GetView(ViewTechnology.Default) as WorkflowView;
            this.workflowView.Dock = System.Windows.Forms.DockStyle.Fill;
            
            this.panel.Child = this.workflowView;

            ISelectionService selectionService = GetService(typeof(ISelectionService)) as ISelectionService;
            if (selectionService != null)
                selectionService.SelectionChanged += new EventHandler(OnSelectionChanged);
        
        }

        public void Save(string file)
        {
            if (this.loader != null)
            {
                this.loader.Xoml = file;
                this.loader.Flush();
            }
        }

        private void OnSelectionChanged(object sender, EventArgs e)
        {
            ISelectionService selectionService = GetService(typeof(ISelectionService)) as ISelectionService;
            if (selectionService != null)
            {
                
            }

        }

        #region IServiceProvider Membres

        public object GetService(Type serviceType)
        {
            if (this.designSurface == null)
                return null;

            return ((IServiceProvider)this.designSurface).GetService(serviceType);
        }

        #endregion


    }
}
