﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Workflow.ComponentModel.Design;
using System.ComponentModel.Design;
using System.Workflow.ComponentModel;
using System.Workflow.Activities;
using System.ComponentModel.Design.Serialization;
using System.Workflow.ComponentModel.Compiler;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.IO;
using System.Xml;
using System.Workflow.ComponentModel.Serialization;
using System.CodeDom;
using System.Diagnostics;

namespace WorkflowDesigner.Workflow
{
    internal class WorkflowLoader : WorkflowDesignerLoader
    {

        /// <summary>
        /// Obtient ou défini le nom du fichier XAML de définition du worflow
        /// </summary>
        public string Xoml
        {
            get;
            set;
        }

        public CompilerResults CompilerResults
        {
            get;
            private set;
        }

        #region Fonctions publiques

        public override string FileName
        {
            get {
                return string.Empty;
            }
        }

        public override System.IO.TextReader GetFileReader(string filePath)
        {
            return null;   
        }

        public override System.IO.TextWriter GetFileWriter(string filePath)
        {
            return null;
        }

        #endregion

        protected override void Initialize()
        {
            base.Initialize();

            IDesignerLoaderHost host = LoaderHost;
            if (host != null)
            {
                
                //Ajout du service de gestion des commandes
                WorkflowMenuCommandService commandService = new WorkflowMenuCommandService(host);
                commandService.Loader = this;
                host.AddService(typeof(IMenuCommandService), commandService);

                TypeProvider typeProvider = new TypeProvider(host);
                host.AddService(typeof(ITypeProvider), typeProvider, true);
            }
        }

        public override void BeginLoad(System.ComponentModel.Design.Serialization.IDesignerLoaderHost host)
        {
            base.BeginLoad(host);
        }

        protected override void PerformLoad(System.ComponentModel.Design.Serialization.IDesignerSerializationManager serializationManager)
        {
            base.PerformLoad(serializationManager);

            //Obtention du designer associé à ce loader
            IDesignerHost designerHost = (IDesignerHost)GetService(typeof(IDesignerHost));

            if (designerHost == null)
                throw new Exception("Impossible d'obtenir le service de design");

            Activity rootActivity = new SequentialWorkflowActivity();
            designerHost.Container.Add(rootActivity, "root");

            designerHost.Activate();
        }

        protected override void PerformFlush(IDesignerSerializationManager serializationManager)
        {
            //base.PerformFlush(serializationManager);
            IDesignerHost host = (IDesignerHost)GetService(typeof(IDesignerHost));
            Activity rootActivity = host.RootComponent as Activity;

            if (host != null && host.RootComponent != null)
            {

                if (rootActivity != null)
                {
                    XmlTextWriter xmlWriter = new XmlTextWriter(this.Xoml, Encoding.Default);
                    try
                    {
                        WorkflowMarkupSerializer xomlSerializer = new WorkflowMarkupSerializer();
                        xomlSerializer.Serialize(xmlWriter, rootActivity);
                    }
                    finally
                    {
                        xmlWriter.Close();
                    }

                    XmlDocument doc = new XmlDocument();
                    doc.Load(this.Xoml);
                    XmlAttribute att = doc.CreateAttribute("Class", "http://schemas.microsoft.com/winfx/2006/xaml");
                    att.Value = "foo.Workflow1";
                    XmlNode node = doc.FirstChild;
                    node.Attributes.Append(att);

                    doc.Save(this.Xoml);
                }
            }

            string codeBesideFile = Path.Combine(Path.GetDirectoryName(this.Xoml), Path.GetFileNameWithoutExtension(this.Xoml) + ".xoml.cs");
            string libFile = Path.Combine(Path.GetDirectoryName(this.Xoml), Path.GetFileNameWithoutExtension(this.Xoml) + ".dll");

            //Création du générateur de code C#
            CSharpCodeProvider provider = new CSharpCodeProvider();

            //Création des options du générateur de code C#
            CodeGeneratorOptions options = new CodeGeneratorOptions();
            options.BlankLinesBetweenMembers = true;
            options.BracingStyle = "C";
            options.ElseOnClosing = false;
            options.IndentString = "    ";

            CodeCompileUnit codeCompileUnit = new CodeCompileUnit();
            codeCompileUnit.ReferencedAssemblies.Add("System.Drawing.dll");
            codeCompileUnit.ReferencedAssemblies.Add("System.dll");

            //Les lib du Fx 3.0 ne sont pas automatiquement reconnues par le framework :-(
            string programFilesPath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles);
            codeCompileUnit.ReferencedAssemblies.Add(
                programFilesPath + @"\Reference Assemblies\Microsoft\Framework\v3.0\System.Workflow.Activities.dll");
            codeCompileUnit.ReferencedAssemblies.Add(
                programFilesPath + @"\Reference Assemblies\Microsoft\Framework\v3.0\System.Workflow.ComponentModel.dll");
            codeCompileUnit.ReferencedAssemblies.Add(
                programFilesPath + @"\Reference Assemblies\Microsoft\Framework\v3.0\System.Workflow.Runtime.dll");

            CodeNamespace ns = new CodeNamespace("foo");
            ns.Imports.Add(new CodeNamespaceImport("System"));
            ns.Imports.Add(new CodeNamespaceImport("System.Drawing"));
            ns.Imports.Add(new CodeNamespaceImport("System.Collections"));
            ns.Imports.Add(new CodeNamespaceImport("System.ComponentModel"));
            ns.Imports.Add(new CodeNamespaceImport("System.ComponentModel.Design"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.ComponentModel.Design"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.ComponentModel"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.ComponentModel.Serialization"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.ComponentModel.Compiler"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.Activities"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.ComponentModel"));
            ns.Imports.Add(new CodeNamespaceImport("System.Workflow.Runtime"));
            codeCompileUnit.Namespaces.Add(ns);

            CodeTypeDeclaration ctd = new CodeTypeDeclaration("Workflow1");
            ctd.BaseTypes.Add("System.Workflow.Activities.SequentialWorkflowActivity");
            ctd.IsPartial = true;
            ctd.Attributes = MemberAttributes.Public;
            ns.Types.Add(ctd);

            //Ecriture du code .NET du workflow
            using (StreamWriter writer = new StreamWriter(codeBesideFile))
            {
                provider.GenerateCodeFromCompileUnit(codeCompileUnit, writer, options);
                writer.Close();
            }

            Process process = new Process();
            process.StartInfo.FileName = @"C:\Windows\Microsoft.NET\Framework\v3.5\MSBuild.exe";
            process.StartInfo.Arguments = string.Format("/tv:3.5 {0}", Path.Combine(Path.GetDirectoryName(this.Xoml), "TestActivityLibrary.csproj"));
            process.StartInfo.RedirectStandardOutput = true;
            process.StartInfo.UseShellExecute = false;

            process.Start();
            process.WaitForExit();

            if (process.ExitCode != 0)
            {
                System.Windows.MessageBox.Show(process.StandardOutput.ReadToEnd());
            }
        }
    }
}
