﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Design;
using System.Windows.Forms;
using System.Workflow.ComponentModel.Design;
using System.Drawing;
using System.CodeDom.Compiler;

namespace WorkflowDesigner.Workflow
{
    /// <summary>
    /// 
    /// </summary>
    public class WorkflowMenuCommandService : MenuCommandService
    {

        /// <summary>
        /// Constructeur par défaut
        /// </summary>
        /// <param name="serviceProvider"></param>
        public WorkflowMenuCommandService(IServiceProvider serviceProvider)
            : base(serviceProvider)
        {
           
        }

        internal WorkflowLoader Loader
        {
            get;
            set;
        }

        public override void ShowContextMenu(CommandID menuID, int x, int y)
        {
            base.ShowContextMenu(menuID, x, y);

            MenuItem[] menuItems = null;
            if (menuID == WorkflowMenuCommands.SelectionMenu)
            {
                menuItems = GetSelectionMenuItems();
            }

            if (menuItems != null)
            {
                ContextMenu contextMenu = new ContextMenu();
                contextMenu.MenuItems.AddRange(menuItems);

                WorkflowView workflowView = GetService(typeof(WorkflowView)) as WorkflowView;

                if (workflowView != null)
                    contextMenu.Show(workflowView, workflowView.PointToClient(new Point(x, y)));
            }
        }

        private MenuItem[] GetSelectionMenuItems()
        {
            Dictionary<CommandID, string> selectionCommands = new Dictionary<CommandID, string>();
            selectionCommands.Add(WorkflowMenuCommands.Cut, "Cut");
            selectionCommands.Add(WorkflowMenuCommands.Copy, "Copy");
            selectionCommands.Add(WorkflowMenuCommands.Paste, "Paste");
            selectionCommands.Add(WorkflowMenuCommands.Delete, "Delete");
            selectionCommands.Add(new CommandID(Guid.NewGuid(), -1), "-");
            selectionCommands.Add(WorkflowMenuCommands.Expand, "Développer");
            selectionCommands.Add(WorkflowMenuCommands.Collapse, "Réduire");
            selectionCommands.Add(new CommandID(Guid.NewGuid(), -1), "-");
            selectionCommands.Add(WorkflowMenuCommands.Print, "Imprimer");

            List<MenuItem> menuItems = new List<MenuItem>();
            
            foreach (CommandID id in selectionCommands.Keys)
            {
                  MenuCommand command = FindCommand(id);

                  if (command != null)
                  {
                      MenuItem menuItem = new MenuItem(selectionCommands[id], new EventHandler(OnMenuClicked));
                      menuItem.Tag = command;
                      menuItems.Add(menuItem);
                  }
                  else
                  {
                      menuItems.Add(new MenuItem("-"));
                  }
                
            }

            menuItems.Add(new MenuItem("Générer un assembly à partir du workflow", new EventHandler(OnGenerateAssembly)));

            return menuItems.Count != 0 ? menuItems.ToArray() : null;
        }

        private void OnMenuClicked(object sender, EventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                MenuCommand command = menuItem.Tag as MenuCommand;

                if (command == FindCommand(WorkflowMenuCommands.Print))
                {
                    WorkflowView workflowView = GetService(typeof(WorkflowView)) as WorkflowView;
                    if (workflowView == null)
                        return;

                    PrintPreviewDialog ppd = new PrintPreviewDialog();
                    ppd.Document = workflowView.PrintDocument;

                    ppd.ShowDialog();
                    return;
                }

                if (command != null)
                    command.Invoke();
            }
        }

        private void OnGenerateAssembly(object sender, EventArgs e)
        {
            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.Filter = "Fichier Xoml|*.xoml";

                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    this.Loader.Xoml = saveFileDialog.FileName;
                    this.Loader.Flush();
                    if (this.Loader.CompilerResults != null)
                    {
                        StringBuilder sb = new StringBuilder();
                        foreach (CompilerError error in this.Loader.CompilerResults.Errors)
                        {
                            sb.Append("Error : "); sb.Append(error.ToString()); sb.Append(Environment.NewLine);
                        }

                        MessageBox.Show(sb.ToString());
                    }
                }
            }
        }
    }
}
