﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Workflow.ComponentModel;
using System.Collections.ObjectModel;
using System.ComponentModel.Design.Serialization;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace WorkflowDesigner.Workflow
{
    /// <summary>
    /// Logique d'interaction pour WorkflowToolbox.xaml
    /// </summary>
    public partial class WorkflowToolbox : UserControl
    {
        private const string CF_DESIGNER = "CF_WINOEDESIGNERCOMPONENTS";
        private ObservableCollection<Activity> activities;
        private WorkflowDesigner designer;

        public WorkflowToolbox()
        {
            InitializeComponent();

            this.activities = new ObservableCollection<Activity>();
            this.listView.ItemsSource = this.activities;
            
            this.listView.MouseMove += new MouseEventHandler(listView_MouseMove);
        }

        public WorkflowDesigner WorkflowDesginer
        {
            get
            {
                return this.designer;
            }

            set
            {
                this.designer = value;
            }
        }

        public void AddActivity(Activity activity)
        {
            if (!this.activities.Contains(activity))
                this.activities.Add(activity);
        }

        private void listView_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.LeftButton == MouseButtonState.Pressed)
            {
                Activity activity = this.listView.SelectedItem as Activity;
                if (activity == null)
                    return;

                IDataObject dataObject = SerializeActivitiesToDataObject(designer, new Activity[] { activity });
                DragDrop.DoDragDrop(this.listView, dataObject, DragDropEffects.Copy | DragDropEffects.Move);
            }
        }

        private static IDataObject SerializeActivitiesToDataObject(IServiceProvider serviceProvider, IEnumerable<Activity> activities)
        {
            // get component serialization service
            ComponentSerializationService css = (ComponentSerializationService)serviceProvider.GetService(typeof(ComponentSerializationService));
            if (css == null)
                throw new InvalidOperationException("Component Serialization Service is missing.");

            
            // serialize all activities to the store
            SerializationStore store = css.CreateStore();
            using (store)
            {
                foreach (Activity activity in activities)
                    css.Serialize(store, activity);
            }

            // wrap it with clipboard style object
            Stream stream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();
            formatter.Serialize(stream, store);
            stream.Seek(0, SeekOrigin.Begin);

            
            return new DataObject(CF_DESIGNER, stream);
        }

    }
}
